/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

// Known MIME types as listed in nsMimeTypes.h
const kKnownImageMIMETypes = [
  "image/gif",
  "image/jpeg",
  "image/jpg",
  "image/pjpeg",
  "image/png",
  "image/apng",
  "image/x-png",
  "image/x-portable-pixmap",
  "image/x-xbitmap",
  "image/x-xbm",
  "image/xbm",
  "image/x-jg",
  "image/tiff",
  "image/bmp",
  "image/x-ms-bmp",
  "image/x-ms-clipboard-bmp",
  "image/x-icon",
  "image/vnd.microsoft.icon",
  "image/icon",
  "video/x-mng",
  "image/x-jng",
  "image/svg+xml",
  "image/webp",
  "image/avif",
  "image/jxl",
  "image/heic",
  "image/heif",
];
